
#ifndef KPUBLICTRANSPORT_EXPORT_H
#define KPUBLICTRANSPORT_EXPORT_H

#ifdef KPUBLICTRANSPORT_STATIC_DEFINE
#  define KPUBLICTRANSPORT_EXPORT
#  define KPUBLICTRANSPORT_NO_EXPORT
#else
#  ifndef KPUBLICTRANSPORT_EXPORT
#    ifdef KPublicTransport_EXPORTS
        /* We are building this library */
#      define KPUBLICTRANSPORT_EXPORT __attribute__((visibility("default")))
#    else
        /* We are using this library */
#      define KPUBLICTRANSPORT_EXPORT __attribute__((visibility("default")))
#    endif
#  endif

#  ifndef KPUBLICTRANSPORT_NO_EXPORT
#    define KPUBLICTRANSPORT_NO_EXPORT __attribute__((visibility("hidden")))
#  endif
#endif

#ifndef KPUBLICTRANSPORT_DEPRECATED
#  define KPUBLICTRANSPORT_DEPRECATED __attribute__ ((__deprecated__))
#endif

#ifndef KPUBLICTRANSPORT_DEPRECATED_EXPORT
#  define KPUBLICTRANSPORT_DEPRECATED_EXPORT KPUBLICTRANSPORT_EXPORT KPUBLICTRANSPORT_DEPRECATED
#endif

#ifndef KPUBLICTRANSPORT_DEPRECATED_NO_EXPORT
#  define KPUBLICTRANSPORT_DEPRECATED_NO_EXPORT KPUBLICTRANSPORT_NO_EXPORT KPUBLICTRANSPORT_DEPRECATED
#endif

#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef KPUBLICTRANSPORT_NO_DEPRECATED
#    define KPUBLICTRANSPORT_NO_DEPRECATED
#  endif
#endif

#endif /* KPUBLICTRANSPORT_EXPORT_H */
