// This file was generated by ecm_setup_version(): DO NOT EDIT!

#ifndef KPUBLICTRANSPORT_VERSION_H
#define KPUBLICTRANSPORT_VERSION_H

#define KPUBLICTRANSPORT_VERSION_STRING "23.08.5"
#define KPUBLICTRANSPORT_VERSION_MAJOR 23
#define KPUBLICTRANSPORT_VERSION_MINOR 8
#define KPUBLICTRANSPORT_VERSION_PATCH 5
#define KPUBLICTRANSPORT_VERSION ((23<<16)|(8<<8)|(5))

#endif
